use crate::filter_expr::value::ValueOrRef;

use super::{expr::Variable, Error, Filter, Value};
use std::collections::HashMap;

fn testsuite<K: Into<Variable>, V: Into<Value>, R: Into<Value>>(
    code: &str,
    external_variables: Vec<(K, V)>,
    ret: Result<R, Error>,
) {
    let filter = Filter::new(code).expect("valid filter");
    let external_variables = external_variables
        .into_iter()
        .map(|(k, v)| (k.into(), ValueOrRef::Value(v.into())))
        .collect();
    assert_eq!(filter.execute(&external_variables), ret.map(|x| x.into()));
}

#[test]
fn test_1() {
    testsuite(
        include_str!("program1.expr"),
        vec![("external", 5)],
        Ok(true),
    );
    testsuite(
        include_str!("program1.expr"),
        vec![("external", 4)],
        Ok(false),
    );
}

#[test]
fn nested() {
    testsuite(
        include_str!("nested.expr"),
        vec![("foo", 0), ("bar", 0), ("five", 5)],
        Ok(false),
    );
    testsuite(
        include_str!("nested.expr"),
        vec![("foo", 1192844), ("bar", 1), ("five", 4)],
        Ok(false),
    );
    testsuite(
        include_str!("nested.expr"),
        vec![("foo", 1192844), ("bar", 1), ("five", 5)],
        Ok(true),
    );
}
